/********************************************
* Object Orientated NeHeGL Using Base Class *
* Author: Andreas Oberdorfer           2004 *
********************************************/

#ifndef	__GL_WINDOW_H__
#define	__GL_WINDOW_H__


#include <windows.h>													// Header File For Windows

#include <gl/gl.h>														// Header File For The OpenGL32 Library
#include <gl/glu.h>														// Header File For The GLu32 Library

class GL_Window															// Window Creation Info
{
public:
	// Class Constructor
	GL_Window();

	// Create OpenGL Window
	bool	Create(const char * window_title, bool full_screen, const char * class_name, HINSTANCE h_instance, LPVOID lpParam);
	// Destroy OpenGL Window
	void	Destroy();
	// Change The Screen Resolution
	bool	ChangeScreenResolution();
	// Reshape The Window When It's Resized
	void	ReshapeGL();
	// Swap Buffers (Double Buffering)
	void	SwapBuffers() { ::SwapBuffers(m_hDC); }
	// Set Window Position Depending On Screen Mode (FullScreen / Windowed)
	void	SetPosX(int x);
	void	SetPosX(unsigned short x) { SetPosX((signed short)x); }		// Sign Extend If Used With WORD Parameter
	void	SetPosY(int y);
	void	SetPosY(unsigned short y) { SetPosY((signed short)y); }		// Sign Extend If Used With WORD Parameter
	// Get Window Dimensions Depending On Screen Mode (FullScreen / Windowed)
	int		GetWidth();
	int		GetHeight();
	// Set Window Dimensions Depending On Screen Mode (FullScreen / Windowed)
	void	SetWidth(int width);
	void	SetHeight(int height);
	// Get Window Position Depending On Screen Mode (FullScreen / Windowed)
	int		GetPosX();
	int		GetPosY();
	// Set BitsPerPixel To 16
	void	SetHiColor()	{ m_BitsPerPixel = 16; }
	// Set BitsPerPixel To 32
	void	SetTrueColor()	{ m_BitsPerPixel = 32; }
	// Class Conversion Operator For Window Handle (GL_Window Can Be Used As A Window Handle)
	operator HWND() { return m_hWnd; }

	HWND	m_hWnd;														// Window Handle
	HDC		m_hDC;														// Device Context
	HGLRC	m_hRC;														// Rendering Context
private:

	int		m_WindowPosX;												// X Position Of Window
	int		m_WindowPosY;												// Y Position Of Window
	int		m_WindowWidth;												// Width Of Window
	int		m_WindowHeight;												// Height Of Window
	int		m_ScreenWidth;												// Width Of FullScreen
	int		m_ScreenHeight;												// Height Of FullScreen
	int		m_BitsPerPixel;												// Bits Per Pixel
	bool	m_IsFullScreen;												// FullScreen
};


#endif	// __GL_WINDOW_H__
