/********************************************
* Object Orientated NeHeGL Using Base Class *
* Author: Andreas Oberdorfer           2004 *
*********************************************************************************
*																				*
*	You Need To Provide The Following Methods:									*
*																				*
*	GL_Application * GL_Application::Create(const char * class_name)			*
*		Create Your Derived Class And Return A GL_Application Pointer			*
*																				*
*	bool Initialize();															*
*		Perform All Your Initialization											*
*		Returns true If Initialization Was Successful, false If Not				*
*																				*
*	void Deinitialize();														*
*		Perform All Your DeInitialization										*
*																				*
*	void Update(DWORD milliseconds);											*
*		Perform Your Motion Updates												*
*		'milliseconds' Is The Number Of Milliseconds Passed Since The Last Call	*
*		With Whatever Accuracy GetTickCount() Provides							*
*																				*
*	void Draw();																*
*		Perform All Your Scene Drawing											*
*																				*
********************************************************************************/

#ifndef	__GL_FRAMEWORK_H__
#define	__GL_FRAMEWORK_H__


#include "GLWindow.h"											// Header File For GLWindow


class Keys														// Class For Keyboard Stuff
{
public:
	// Class Constructor
	Keys() { Clear(); }
	// Clear All Keys
	void Clear() { ZeroMemory(&m_KeyDown, sizeof(m_KeyDown)); }
	// frame
	void frame(void){memcpy(m_OldKeyDown,m_KeyDown,sizeof(bool)*MAX_KEYS);}
	// Return true If The Key State Is Pressed
	bool IsPressed(unsigned int key)	{ return (key < MAX_KEYS) ? (m_KeyDown[key] == true) : false; }
	// returns true if it's being pressed down
	bool isHit(unsigned int key)		{ return (key < MAX_KEYS) ? (m_KeyDown[key] != m_OldKeyDown[key])&&m_KeyDown[key] : false; }
	// Set a Key To State Pressed
	void SetPressed(unsigned int key)	{ if (key < MAX_KEYS) m_KeyDown[key] = true; }
	// Set a Key To State Released
	void SetReleased(unsigned int key)	{ if (key < MAX_KEYS) m_KeyDown[key] = false; }

private:
	static const unsigned int MAX_KEYS = 256;
	bool m_OldKeyDown[MAX_KEYS];								// Holds true / false For last fram and each Key
	bool m_KeyDown[MAX_KEYS];									// Holds true / false For Each Key
};


class GL_Application											// Base Class Contains Information Vital To A Window
{
public:
	// This Method Must Be Provided By Derived Class
	static GL_Application * Create(const char * class_name);	// Create Your Derived Class

	// Virtal Destructor Is Necessary To Avoid Memory Leaks When Deleting The (Derived) Class
	virtual ~GL_Application() {};

protected:
	// These Methods Must Be Provided By Derived Class
	virtual bool	Initialize() = 0;							// Perform All Initialization
	virtual void	Deinitialize() = 0;							// Perform All DeInitialization
	virtual void	Update(DWORD milliseconds) = 0;				// Perform Motion Updates
	virtual void	Draw() = 0;									// Perform All Scene Drawing

	// Common Methods Provided By Base Class
	void	ToggleFullscreen();									// Toggle Fullscreen / Windowed Mode
	void	TerminateApplication();								// Terminate The Application
	void	ResizeDraw(bool enable) { m_ResizeDraw = enable; }	// Enable Redraw During Window Resize
	Keys	m_Keys;												// Key Class

	// Only Derived Classes Are Allowed To Instanciate This Class (e.g. class GL_Example)
	GL_Application(const char * class_name);					// Class Constructor

	GL_Window	m_Window;										// Window Class

private:
	// Program Main Loop
	friend int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow);
	int		Main(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow);
	// Message Handler
	friend LRESULT CALLBACK WindowProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
	LRESULT	Message(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);

	static const UINT WM_TOGGLEFULLSCREEN = (WM_USER + 1);		// Application Define Message For Toggling

	const char * m_ClassName;									// Application ClassName
	bool		m_IsProgramLooping;								// Window Creation Loop, For FullScreen / Windowed Toggle
	bool		m_CreateFullScreen;								// If true, Then Create Fullscreen
	bool		m_IsVisible;									// Application Window Visible
	bool		m_ResizeDraw;									// Call Draw() During Window Resize
	DWORD		m_LastTickCount;								// Last Tick Count
};


#endif	// __GL_FRAMEWORK_H__
